@@ -5,32 +5,20 @@ Functions for running multiple Covasim runs.
 #%% Imports
 import numpy as np
 import pylab as pl
+import pandas as pd
 import sciris as sc
+import datetime as dt
+from collections import defaultdict
+import matplotlib.ticker as ticker
+from . import defaults as cvd
 from . import base as cvbase
 from . import sim as cvsim
 
 
 # Specify all externally visible functions this file defines
-__all__ = ['default_scen_plots', 'default_scenario', 'make_metapars', 'Scenarios', 'single_run', 'multi_run']
+__all__ = ['make_metapars', 'Scenarios', 'single_run', 'multi_run']
 
 
-default_scen_plots = sc.odict({
-            'cum_infections': 'Cumulative infections',
-            # 'cum_deaths': 'Cumulative deaths',
-            # 'cum_recoveries':'Cumulative recoveries',
-            # 'cum_tested': 'Cumulative tested',
-            # 'n_susceptible': 'Number susceptible',
-            'n_infectious': 'Number of active infections',
-            # 'cum_diagnosed': 'Cumulative diagnosed',
-            # 'infections': 'New infections',
-            # 'deaths': 'New deaths',
-            # 'recoveries': 'New recoveries',
-            # 'tests': 'Number of tests',
-            # 'diagnoses': 'New diagnoses',
-    })
-
-default_scenario = {'baseline':{'name':'Baseline', 'pars':{}}}
-
 
 def make_metapars():
     ''' Create default metaparameters for a Scenarios run '''
@@ -38,7 +26,7 @@ def make_metapars():
         n_runs    = 3, # Number of parallel runs; change to 3 for quick, 11 for real
         noise     = 0.1, # Use noise, optionally
         noisepar  = 'beta',
-        seed      = 1,
+        rand_seed = 1,
         quantiles = {'low':0.1, 'high':0.9},
         verbose   = 1,
     )
@@ -75,22 +63,18 @@ class Scenarios(cvbase.ParsObj):
 
         # Handle scenarios -- by default, create a baseline scenario
         if scenarios is None:
-            scenarios = sc.dcp(default_scenario)
+            scenarios = {'baseline':{'name':'Baseline', 'pars':{}}}
         self.scenarios = scenarios
 
         # Handle metapars
-        if metapars is None:
-            metapars = {}
-        self.metapars = metapars
+        self.metapars = sc.mergedicts({}, metapars)
         self.update_pars(self.metapars)
 
         # Create the simulation and handle basepars
         if sim is None:
             sim = cvsim.Sim()
         self.base_sim = sim
-        if basepars is None:
-            basepars = {}
-        self.basepars = basepars
+        self.basepars = sc.mergedicts({}, basepars)
         self.base_sim.update_pars(self.basepars)
         self.base_sim.validate_pars()
         self.base_sim.init_results()
@@ -98,20 +82,29 @@ class Scenarios(cvbase.ParsObj):
         # Copy quantities from the base sim to the main object
         self.npts = self.base_sim.npts
         self.tvec = self.base_sim.tvec
-        self.reskeys = self.base_sim.reskeys
 
         # Create the results object; order is: results key, scenario, best/low/high
         self.sims = sc.objdict()
-        self.allres = sc.objdict()
-        for reskey in self.reskeys:
-            self.allres[reskey] = sc.objdict()
+        self.results = sc.objdict()
+        for reskey in self.result_keys():
+            self.results[reskey] = sc.objdict()
             for scenkey in scenarios.keys():
-                self.allres[reskey][scenkey] = sc.objdict()
+                self.results[reskey][scenkey] = sc.objdict()
                 for nblh in ['name', 'best', 'low', 'high']:
-                    self.allres[reskey][scenkey][nblh] = None # This will get populated below
+                    self.results[reskey][scenkey][nblh] = None # This will get populated below
         return
 
 
+    def result_keys(self):
+        ''' Attempt to retrieve the results keys from the base sim '''
+        try:
+            keys = self.base_sim.result_keys()
+        except Exception as E:
+            errormsg = f'Could not retrieve result keys since base sim not accessible: {str(E)}'
+            raise ValueError(errormsg)
+        return keys
+
+
     def run(self, debug=False, verbose=None, **kwargs):
         '''
         Run the actual scenarios
@@ -136,7 +129,7 @@ class Scenarios(cvbase.ParsObj):
                 print(string)
             return
 
-        reskeys = self.reskeys # Shorten since used extensively
+        reskeys = self.result_keys() # Shorten since used extensively
 
         # Loop over scenarios
         for scenkey,scen in self.scenarios.items():
@@ -152,6 +145,7 @@ class Scenarios(cvbase.ParsObj):
 
             print_heading(f'Multirun for {scenkey}')
             scen_sim = sc.dcp(self.base_sim)
+            scen_sim.label = scenkey
             scen_sim.update_pars(scenpars)
             run_args = dict(n_runs=self['n_runs'], noise=self['noise'], noisepar=self['noisepar'], verbose=verbose)
             if debug:
@@ -175,35 +169,39 @@ class Scenarios(cvbase.ParsObj):
             scenres.low = {}
             scenres.high = {}
             for reskey in reskeys:
-                scenres.best[reskey] = pl.mean(scenraw[reskey], axis=1) # Changed from median to mean for smoother plots
+                scenres.best[reskey] = pl.median(scenraw[reskey], axis=1) # Changed from median to mean for smoother plots
                 scenres.low[reskey]  = pl.quantile(scenraw[reskey], q=self['quantiles']['low'], axis=1)
                 scenres.high[reskey] = pl.quantile(scenraw[reskey], q=self['quantiles']['high'], axis=1)
 
             for reskey in reskeys:
-                self.allres[reskey][scenkey]['name'] = scenname
+                self.results[reskey][scenkey]['name'] = scenname
                 for blh in ['best', 'low', 'high']:
-                    self.allres[reskey][scenkey][blh] = scenres[blh][reskey]
+                    self.results[reskey][scenkey][blh] = scenres[blh][reskey]
 
             self.sims[scenkey] = scen_sims
 
-
-
         #%% Print statistics
         if verbose:
-            print('\nResults for final time point in each scenario:')
-            for reskey in reskeys:
-                print(f'\n{reskey}')
-                for scenkey in list(self.scenarios.keys()):
-                    print(f'  {scenkey}: {self.allres[reskey][scenkey].best[-1]:0.0f}')
-            print() # Add a blank space
+            sc.heading('Results for last day in each scenario:')
+            x = defaultdict(dict)
+            scenkeys = list(self.scenarios.keys())
+            for scenkey in scenkeys:
+                for reskey in reskeys:
+                    val = self.results[reskey][scenkey].best[-1]
+                    if reskey not in ['r_eff', 'doubling_time']:
+                        val = int(val)
+                    x[scenkey][reskey] = val
+            df = pd.DataFrame.from_dict(x).astype(object)
+            print(df)
+            print()
 
         return
 
 
     def plot(self, to_plot=None, do_save=None, fig_path=None, fig_args=None, plot_args=None,
-             axis_args=None, fill_args=None, as_dates=True, interval=None, dateformat=None,
-             font_size=18, font_family=None, grid=True, commaticks=True, do_show=True, sep_figs=False,
-             verbose=None):
+             axis_args=None, fill_args=None, legend_args=None, as_dates=True, dateformat=None,
+             interval=None, n_cols=1, font_size=18, font_family=None, grid=True, commaticks=True,
+             do_show=True, sep_figs=False, verbose=None):
         '''
         Plot the results -- can supply arguments for both the figure and the plots.
 
@@ -215,9 +213,11 @@ class Scenarios(cvbase.ParsObj):
             plot_args   (dict): Dictionary of kwargs to be passed to pl.plot()
             axis_args   (dict): Dictionary of kwargs to be passed to pl.subplots_adjust()
             fill_args   (dict): Dictionary of kwargs to be passed to pl.fill_between()
+            legend_args (dict): Dictionary of kwargs to be passed to pl.legend()
             as_dates    (bool): Whether to plot the x-axis as dates or time points
-            interval    (int):  Interval between tick marks
             dateformat  (str):  Date string format, e.g. '%B %d'
+            interval    (int):  Interval between tick marks
+            n_cols      (int):  Number of columns of subpanels to use for subplot
             font_size   (int):  Size of the font
             font_family (str):  Font face
             grid        (bool): Whether or not to plot gridlines
@@ -235,14 +235,15 @@ class Scenarios(cvbase.ParsObj):
         sc.printv('Plotting...', 1, verbose)
 
         if to_plot is None:
-            to_plot = default_scen_plots
-        to_plot = sc.odict(sc.dcp(to_plot)) # In case it's supplied as a dict
+            to_plot = cvd.default_scen_plots
+        to_plot = sc.dcp(to_plot) # In case it's supplied as a dict
 
         # Handle input arguments -- merge user input with defaults
-        fig_args  = sc.mergedicts({'figsize': (16, 12)}, fig_args)
-        plot_args = sc.mergedicts({'lw': 3, 'alpha': 0.7}, plot_args)
-        axis_args = sc.mergedicts({'left': 0.10, 'bottom': 0.05, 'right': 0.95, 'top': 0.90, 'wspace': 0.5, 'hspace': 0.25}, axis_args)
-        fill_args = sc.mergedicts({'alpha': 0.2}, fill_args)
+        fig_args    = sc.mergedicts({'figsize': (16, 14)}, fig_args)
+        plot_args   = sc.mergedicts({'lw': 3, 'alpha': 0.7}, plot_args)
+        axis_args   = sc.mergedicts({'left': 0.10, 'bottom': 0.05, 'right': 0.95, 'top': 0.90, 'wspace': 0.25, 'hspace': 0.25}, axis_args)
+        fill_args   = sc.mergedicts({'alpha': 0.2}, fill_args)
+        legend_args = sc.mergedicts({'loc': 'best'}, legend_args)
 
         if sep_figs:
             figs = []
@@ -253,15 +254,16 @@ class Scenarios(cvbase.ParsObj):
         if font_family:
             pl.rcParams['font.family'] = font_family
 
-        # %% Plotting
-        for rk,reskey,title in to_plot.enumitems():
+        n_rows = np.ceil(len(to_plot)/n_cols) # Number of subplot rows to have
+        for rk,reskey in enumerate(to_plot):
+            title = self.base_sim.results[reskey].name # Get the name of this result from the base simulation
             if sep_figs:
                 figs.append(pl.figure(**fig_args))
                 ax = pl.subplot(111)
             else:
-                ax = pl.subplot(len(to_plot), 1, rk + 1)
+                ax = pl.subplot(n_rows, n_cols, rk + 1)
 
-            resdata = self.allres[reskey]
+            resdata = self.results[reskey]
 
             for scenkey, scendata in resdata.items():
 
@@ -269,7 +271,7 @@ class Scenarios(cvbase.ParsObj):
                 pl.plot(self.tvec, scendata.best, label=scendata.name, **plot_args)
                 pl.title(title)
                 if rk == 0:
-                    pl.legend(loc='best')
+                    pl.legend(**legend_args)
 
                 pl.grid(grid)
                 if commaticks:
@@ -282,9 +284,12 @@ class Scenarios(cvbase.ParsObj):
 
                 # Set xticks as dates
                 if as_dates:
-                    xticks = ax.get_xticks()
-                    xticklabels = self.base_sim.inds2dates(xticks, dateformat=dateformat)
-                    ax.set_xticklabels(xticklabels)
+                    @ticker.FuncFormatter
+                    def date_formatter(x, pos):
+                        return (self.base_sim['start_day'] + dt.timedelta(days=x)).strftime('%b-%d')
+                    ax.xaxis.set_major_formatter(date_formatter)
+                    if not interval:
+                        ax.xaxis.set_major_locator(ticker.MaxNLocator(integer=True))
 
         # Ensure the figure actually renders or saves
         if do_save:
@@ -301,15 +306,68 @@ class Scenarios(cvbase.ParsObj):
         return fig
 
 
+    def to_json(self, filename=None, tostring=True, indent=2, verbose=False, *args, **kwargs):
+        '''
+        Export results as JSON.
 
-    def save(self, filename=None, keep_sims=True, keep_people=False, **kwargs):
+        Args:
+            filename (str): if None, return string; else, write to file
+
+        Returns:
+            A unicode string containing a JSON representation of the results,
+            or writes the JSON file to disk
+
+        '''
+        d = {'t':self.tvec,
+             'results':   self.results,
+             'basepars':  self.basepars,
+             'metapars':  self.metapars,
+             'simpars':   self.base_sim._make_pardict(),
+             'scenarios': self.scenarios
+             }
+        if filename is None:
+            output = sc.jsonify(d, tostring=tostring, indent=indent, verbose=verbose, *args, **kwargs)
+        else:
+            output = sc.savejson(filename=filename, obj=d, indent=indent, *args, **kwargs)
+
+        return output
+
+
+    def to_excel(self, filename=None):
+        '''
+        Export results as XLSX
+
+        Args:
+            filename (str): if None, return string; else, write to file
+
+        Returns:
+            An sc.Spreadsheet with an Excel file, or writes the file to disk
+
+        '''
+        spreadsheet = sc.Spreadsheet()
+        spreadsheet.freshbytes()
+        with pd.ExcelWriter(spreadsheet.bytes, engine='xlsxwriter') as writer:
+            for key in self.result_keys():
+                result_df = pd.DataFrame.from_dict(sc.flattendict(self.results[key], sep='_'))
+                result_df.to_excel(writer, sheet_name=key)
+        spreadsheet.load()
+
+        if filename is None:
+            output = spreadsheet
+        else:
+            output = spreadsheet.save(filename)
+
+        return output
+
+
+    def save(self, filename=None, keep_sims=True, keep_population=False, **kwargs):
         '''
         Save to disk as a gzipped pickle.
 
         Args:
             filename (str or None): the name or path of the file to save to; if None, uses stored
             keep_sims (bool): whether or not to store the actual Sim objects in the Scenarios object
-            keep_people (bool): whether or not to store the people in the Sim objects (NB, very large)
+            keep_population (bool): whether or not to store the population in the Sim objects (NB, very large)
             keywords: passed to makefilepath()
 
         Returns:
@@ -331,7 +389,7 @@ class Scenarios(cvbase.ParsObj):
         obj = sc.dcp(self) # This should be quick once we've removed the sims
 
         if keep_sims:
-            if keep_people:
+            if keep_population:
                 obj.sims = sims # Just restore the object in full
                 print('Note: saving people, which may produce a large file!')
             else:
@@ -339,7 +397,7 @@ class Scenarios(cvbase.ParsObj):
                 for key in sims.keys():
                     obj.sims[key] = []
                     for sim in sims[key]:
-                        obj.sims[key].append(sim.shrink())
+                        obj.sims[key].append(sim.shrink(in_place=False))
 
         sc.saveobj(filename=filename, obj=obj) # Actually save
 
@@ -368,7 +426,7 @@ class Scenarios(cvbase.ParsObj):
 
 
 
-def single_run(sim, ind=0, noise=0.0, noisepar=None, verbose=None, run_args=None, sim_args=None, **kwargs):
+def single_run(sim, ind=0, noise=0.0, noisepar=None, verbose=None, keep_people=False, run_args=None, sim_args=None, **kwargs):
     '''
     Convenience function to perform a single simulation run. Mostly used for
     parallelization, but can also be used directly.
@@ -400,7 +458,7 @@ def single_run(sim, ind=0, noise=0.0, noisepar=None, verbose=None, run_args=None
     sim_args = sc.mergedicts(sim_args, kwargs)
     run_args = sc.mergedicts({'verbose':verbose}, run_args)
 
-    new_sim['seed'] += ind # Reset the seed, otherwise no point of parallel runs
+    new_sim['rand_seed'] += ind # Reset the seed, otherwise no point of parallel runs
     new_sim.set_seed()
 
     # If the noise parameter is not found, guess what it should be
@@ -418,7 +476,7 @@ def single_run(sim, ind=0, noise=0.0, noisepar=None, verbose=None, run_args=None
     new_sim[noisepar] *= noisefactor
 
     if verbose>=1:
-        print(f'Running a simulation using {new_sim["seed"]} seed and {noisefactor} noise')
+        print(f'Running a simulation using {new_sim["rand_seed"]} seed and {noisefactor} noise')
 
     # Handle additional arguments
     for key,val in sim_args.items():
@@ -433,10 +491,14 @@ def single_run(sim, ind=0, noise=0.0, noisepar=None, verbose=None, run_args=None
     # Run
     new_sim.run(**run_args)
 
+    # Shrink the sim to save memory
+    if not keep_people:
+        new_sim.shrink()
+
     return new_sim
 
 
-def multi_run(sim, n_runs=4, noise=0.0, noisepar=None, iterpars=None, verbose=None, combine=False, run_args=None, sim_args=None, **kwargs):
+def multi_run(sim, n_runs=4, noise=0.0, noisepar=None, iterpars=None, verbose=None, combine=False, keep_people=None, run_args=None, sim_args=None, **kwargs):
     '''
     For running multiple runs in parallel.
 
@@ -448,6 +510,7 @@ def multi_run(sim, n_runs=4, noise=0.0, noisepar=None, iterpars=None, verbose=No
         iterpars (dict): any other parameters to iterate over the runs; see sc.parallelize() for syntax
         verbose (int): detail to print
         combine (bool): whether or not to combine all results into one sim, rather than return multiple sim objects
+        keep_people (bool): whether or not to keep the people in each sim
         run_args (dict): arguments passed to sim.run()
         sim_args (dict): extra parameters to pass to the sim
         kwargs (dict): also passed to the sim
@@ -483,7 +546,7 @@ def multi_run(sim, n_runs=4, noise=0.0, noisepar=None, iterpars=None, verbose=No
     # Copy the simulations
     iterkwargs = {'ind':np.arange(n_runs)}
     iterkwargs.update(iterpars)
-    kwargs = {'sim':sim, 'noise':noise, 'noisepar':noisepar, 'verbose':verbose, 'sim_args':sim_args, 'run_args':run_args}
+    kwargs = {'sim':sim, 'noise':noise, 'noisepar':noisepar, 'verbose':verbose, 'keep_people':keep_people, 'sim_args':sim_args, 'run_args':run_args}
     sims = sc.parallelize(single_run, iterkwargs=iterkwargs, kwargs=kwargs)
 
     # Usual case -- return a list of sims
@@ -493,16 +556,18 @@ def multi_run(sim, n_runs=4, noise=0.0, noisepar=None, iterpars=None, verbose=No
     # Or, combine them into a single sim with scaled results
     else:
         output_sim = sc.dcp(sims[0])
-        output_sim.pars['parallelized'] = n_runs # Store how this was parallelized
-        output_sim.pars['n'] *= n_runs # Restore this since used in later calculations -- a bit hacky, it's true
+        output_sim.parallelized = {'parallelized':True, 'combined':True, 'n_runs':n_runs}  # Store how this was parallelized
+        output_sim['pop_size'] *= n_runs  # Record the number of people
+
         for s,sim in enumerate(sims[1:]): # Skip the first one
-            output_sim.people.update(sim.people)
-            for key in sim.reskeys:
+            if keep_people:
+                output_sim.people += sim.people
+            for key in sim.result_keys():
                 this_res = sim.results[key]
                 output_sim.results[key].values += this_res.values
 
         # For non-count results (scale=False), rescale them
-        for key in output_sim.reskeys:
+        for key in output_sim.result_keys():
             if not output_sim.results[key].scale:
                 output_sim.results[key].values /= len(sims)
 
