@@ -16,11 +16,10 @@ scenarios = {
 }
 
 # Run options
-do_run = 1
+do_run  = 1
 do_save = 0 # refers to whether to save plot - see also save_sims
 do_plot = 1
 do_show = 1
-keep_sims = 0 # WARNING, huge! (>100 MB)
 verbose = 1
 
 # Sim options
@@ -29,14 +28,14 @@ interv_eff = 0.7
 default_beta = 0.015 # Should match parameters.py
 
 basepars = dict(
-  n = 5000
+  pop_size = 5000
 )
 
 metapars = dict(
-    n_runs = 3, # Number of parallel runs; change to 3 for quick, 11 for real
-    noise = 0.1, # Use noise, optionally
-    noisepar = 'beta',
-    seed = 1,
+    n_runs    = 3, # Number of parallel runs; change to 3 for quick, 11 for real
+    noise     = 0.1, # Use noise, optionally
+    noisepar  = 'beta',
+    rand_seed = 1,
     quantiles = {'low':0.1, 'high':0.9},
 )
 
@@ -45,8 +44,8 @@ version  = 'v0'
 date     = '2020mar24'
 folder   = 'results'
 basename = f'{folder}/covasim_scenarios_{date}_{version}'
-fig_path   = f'{basename}.png'
-obj_path   = f'{basename}.scens'
+fig_path = f'{basename}.png'
+obj_path = f'{basename}.scens'
 
 # Define the scenarios
 scenarios = {'baseline': {
@@ -61,12 +60,12 @@ scenarios = {'baseline': {
                   'interventions': cv.change_beta(days=interv_day, changes=interv_eff)
                   }
               },
-            # 'distance2': { # With noise = 0.0, this should be identical to the above
-            #   'name':'Social distancing, version 2',
-            #   'pars': {
-            #       'interventions': cv.dynamic_pars({'beta':dict(days=interv_day, vals=interv_eff*default_beta)})
-            #       }
-            #   },
+            'distance2': { # With noise = 0.0, this should be identical to the above
+              'name':'Social distancing, version 2',
+              'pars': {
+                  'interventions': cv.dynamic_pars({'beta':dict(days=interv_day, vals=interv_eff*default_beta)})
+                  }
+              },
              }
 
 
@@ -79,7 +78,7 @@ if __name__ == "__main__": # Required for parallel processing on Windows
         scens = cv.Scenarios(basepars=basepars, metapars=metapars, scenarios=scenarios)
         scens.run(verbose=verbose)
         if do_save:
-            scens.save(filename=obj_path, keep_sims=keep_sims)
+            scens.save(filename=obj_path)
 
     # Don't run
     else:
