@@ -17,19 +17,19 @@ interv  = 0
 
 # Set filename if saving
 version  = 'v0'
-date     = '2020apr05'
+date     = '2020apr15'
 folder   = 'results'
 basename = f'{folder}/covasim_run_{date}_{version}'
 fig_path = f'{basename}.png'
 
 # Configure the sim -- can also just use a normal dictionary
 pars = sc.objdict(
-    n           = 20000, # Population size
-    n_infected  = 1,    # Number of initial infections
-    n_days      = 180,   # Number of days to simulate
-    prog_by_age = 1,    # Use age-specific mortality etc.
-    usepopdata  = 1,    # Use realistic population structure (requires synthpops)
-    seed        = 1,    # Random seed
+    pop_size     = 20000, # Population size
+    pop_infected = 1,     # Number of initial infections
+    n_days       = 60,   # Number of days to simulate
+    rand_seed    = 1,     # Random seed
+    pop_type     = 'random',
+    use_layers   = True,
     )
 
 # Optionally add an intervention
@@ -45,13 +45,3 @@ sim.run(verbose=verbose)
 if do_plot:
     print('Plotting...')
     fig = sim.plot(do_save=do_save, do_show=do_show, fig_path=fig_path)
-
-
-
-
-
-
-
-
-
-
